==> Scientific.0.8.9.tar.gz: FBEnterprise.framework/Versions/A/Headers/FBQualifier.h <==
/* FBQualifier.h created by ahoesch on Tue 26-Sep-2000 */

#import <Foundation/Foundation.h>
#import "FBEntity.h"
#import "FBRelationship.h"
#import "FBObject.h"

@class FBSQLExpression;
@class FBQualifier;

// Constants so users will get compiler warnings if they make a spelling error
#define FBQualifierOperatorEqual @selector(isEqualTo:)
#define FBQualifierOperatorNotEqual @selector(isNotEqualTo:)
#define FBQualifierOperatorLessThan @selector(isLessThan:)
#define FBQualifierOperatorGreaterThan @selector(isGreaterThan:)
#define FBQualifierOperatorLessThanOrEqualTo @selector(isLessThanOrEqualTo:)
#define FBQualifierOperatorGreaterThanOrEqualTo @selector(isGreaterThanOrEqualTo:)
#define FBQualifierOperatorContains @selector(doesContain:)
#define FBQualifierOperatorLike @selector(isLike:)

typedef BOOL (*BOOL_IMP)(id, SEL, ...);

// All qualifiers that can be evaluated in memory implement this protocol
@protocol FBQualifierEvaluation

- (BOOL)evaluateWithObject:(FBObject *)object; // returns YES if argument satifies the qualifier, NO otherwise.

@end

// All qualifiers that can be used to generate SQL queries implement this protocol
@protocol FBQualifierSQLGeneration

- (NSString *)sqlStringForSQLExpression:(FBSQLExpression *)sqlExpression;

- (FBQualifier *)schemaBasedQualifierWithRootEntity:(FBEntity *)entity;
// Returns an equivalent qualifier with object references
// replaced by foreign key references.

@end

// The following selectors are used when parsing an FBQualifier format.
// The following methods correspond to the following operators: =, <=, <, >=, >,
// !=, contains, and like.
/*@interface NSObject (FBRelationalSelectors)
- (BOOL)isEqualTo:(id)object;
    // Implemented using isEqual:. Returns NO if receiever is nil.
- (BOOL)isLessThanOrEqualTo:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)isLessThan:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)isGreaterThanOrEqualTo:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)isGreaterThan:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)isNotEqualTo:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)doesContain:(id)object;
    // Returns nil if receiver is not an NSArray or if array doesn't contain object.
- (BOOL)isLike:(NSString *)object;
    // argument should be a string using simple shell wildcards (* and ?).
    // (e.g. "Stev*" or "N?XT").
    // Returns NO if receiver is not an NSString.
@end*/

@interface FBEntity (QualifierGeneration)

- (FBQualifier *)qualifierForObject:(FBObject *)object;
- (FBQualifier *)qualifierForGlobalID:(FBGlobalID *)globalID;

@end

@interface FBRelationship (QualifierGeneration)

- (FBQualifier *)destinationQualififerForObject:(FBObject *)object;

@end

/* **************************************************
   FBQualifier
   ************************************************** */
@interface FBQualifier : NSObject <FBQualifierEvaluation>
{
}

+ (NSDictionary *)selectorDic;
+ (FBQualifier *)qualifierWithQualifierString:(NSString *)string entity:(FBEntity *)entity;

// Used for parsing and printing of qualifiers
+ (NSString *)stringForOperatorSelector:(SEL)selector;
+ (SEL)operatorSelectorForString:(NSString *)string;

- (NSString *)sqlStringForSQLExpression:(FBSQLExpression *)sqlExpression;
- (BOOL)isComplexKeyQualifier;

@end

/* **************************************************
   FBKeyValueQualifier
   ************************************************** */
@interface FBKeyValueQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   SEL _selector;
   NSString *_key;
   id _value;
   BOOL _casesensitive;
}

+ (id)qualifierWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value;
+ (id)qualifierWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value casesensitive:(BOOL)flag;

- (id)initWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value;
- (id)initWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value casesensitive:(BOOL)flag;

- (BOOL)isCasesensitive;

- (SEL)selector;

- (NSString *)key;

- (id)value;

@end

/* **************************************************
   FBKeyArrayQualifier
   ************************************************** */
@interface FBKeyArrayQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   BOOL _all;
   NSString *_key;
   NSArray *_array;
}

+ (id)qualifierWithKey:(NSString *)key and:(BOOL)flag array:(NSArray *)array;

- (id)initWithKey:(NSString *)key and:(BOOL)flag array:(NSArray *)array;

- (BOOL)all;

- (NSString *)key;

- (NSArray *)array;

@end

/* **************************************************
   FBSQLQualifier
   ************************************************** */
@interface FBSQLQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSString *_sql;
}

+ (id)qualifierWithSQL:(NSString *)sql;
- (id)initWithSQL:(NSString *)sql;

- (NSString *)sql;

@end

/* **************************************************
   FBInQualifier
   ************************************************** */
@interface FBInQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSString *_key;
   NSArray *_array;
}

+ (id)qualifierWithKey:(NSString *)key array:(NSArray *)array;

- (id)initWithKey:(NSString *)key array:(NSArray *)array;

- (NSString *)key;

- (NSArray *)array;

@end

/* **************************************************
   FBNotInQualifier
   ************************************************** */
@interface FBNotInQualifier : FBInQualifier
{
}

@end

/* **************************************************
   FBKeyComparisonQualifier
   ************************************************** */
@interface FBKeyComparisonQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
    SEL _selector;
    NSString *_leftKey;
    NSString *_rightKey;
}

+ (id)qualifierWithLeftKey:(NSString *)leftKey operatorSelector:(SEL)selector rightKey:(NSString *)rightKey;

- (id)initWithLeftKey:(NSString *)leftKey operatorSelector:(SEL)selector rightKey:(id)rightKey;

- (SEL)selector;

- (NSString *)leftKey;

- (NSString *)rightKey;

@end

/* **************************************************
   FBKeyIsNullQualifier
   ************************************************** */
@interface FBKeyIsNullQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSString *_key;
}

+ (id)qualifierWithKey:(NSString *)key;
- (id) initWithKey:(NSString *)key;
- (NSString *)key;

@end

/* **************************************************
   FBFalseQualifier
   ************************************************** */
@interface FBFalseQualifier : FBKeyIsNullQualifier

+ (id)qualifierForEntity:(FBEntity *)entity;

@end

/* **************************************************
   FBKeyIsNotNullQualifier
   ************************************************** */
@interface FBKeyIsNotNullQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSString *_key;
}

+ (id)qualifierWithKey:(NSString *)key;
- (id)initWithKey:(NSString *)key;
- (NSString *)key;

@end

/* **************************************************
   FBAndQualifier
   ************************************************** */
@interface FBAndQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSMutableArray *_qualifiers;
}

+ (id)qualifierWithQualifierArray:(NSArray *)array;

- (id)initWithQualifierArray:(NSArray *)array;

- (NSArray *)qualifiers;

@end

/* **************************************************
   FBOrQualifier
   ************************************************** */
@interface FBOrQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration> 
{
    NSMutableArray *_qualifiers;
}

+ (id)qualifierWithQualifierArray:(NSArray *)array;

- (id)initWithQualifierArray:(NSArray *)array;

- (NSArray *)qualifiers;

@end

/* **************************************************
   FBKeyIsNullOrZeroQualifier
   ************************************************** */
@interface FBKeyIsNullOrZeroQualifier : FBOrQualifier
{
}

+ (id)qualifierWithKey:(NSString *)key;

@end

/* **************************************************
   FBIntermediateQualifier
   ************************************************** */
@interface FBIntermediateQualifier : FBQualifier <FBQualifierSQLGeneration>
{
   FBRelationship * _relationship;
   NSDictionary *_valueDic;
}

// This method is used by +[FBQualififer destinationQualifierForObject:] to qualify the target objects of a flattened relationship, e.g. Person -- addresses -->> Address over an intermediate table Person_addresses. This class is supposed to generate a where claus of the form "select * from Address where addressId in (select addressId from (Person_addresses natural join Address) where personId = ...)". The points are replaced with value. The joins are generated according to the relationships defined in the flattened relationship.
- (id)initWithFlattenedRelationship:(FBRelationship *)relationship valueDic:(NSDictionary *)valueDic;

- (FBRelationship *)relationship;

- (id)valueForKey:(NSString *)key;

@end

@interface NSString (NSStringQualifierExtension)

// returns YES if opening and closing bracktes summarize to zero
- (BOOL)isBalanced;

// removes leading and tailing spaces and bracktes
- (NSString *)cleanedString;
- (NSString *)stringAfterRemovingLeadingAndTailingSpaces;
- (NSString *)stringAfterRemovingLeadingAndTailingBrackets;

- (SEL)operatorSelectorForLeft:(NSString **)pLeft right:(NSString **)pRight;

- (id)valueForAttribute:(FBAttribute *)attribute;

@end

==> Scientific.0.9.tar.gz: FBEnterprise.framework/Versions/A/Headers/FBQualifier.h <==
/* FBQualifier.h created by ahoesch on Tue 26-Sep-2000 */

#import <Foundation/Foundation.h>
#import "FBEntity.h"
#import "FBRelationship.h"
#import "FBObject.h"

@class FBSQLExpression;
@class FBQualifier;

// Constants so users will get compiler warnings if they make a spelling error
#define FBQualifierOperatorEqual @selector(isEqualTo:)
#define FBQualifierOperatorNotEqual @selector(isNotEqualTo:)
#define FBQualifierOperatorLessThan @selector(isLessThan:)
#define FBQualifierOperatorGreaterThan @selector(isGreaterThan:)
#define FBQualifierOperatorLessThanOrEqualTo @selector(isLessThanOrEqualTo:)
#define FBQualifierOperatorGreaterThanOrEqualTo @selector(isGreaterThanOrEqualTo:)
#define FBQualifierOperatorContains @selector(doesContain:)
#define FBQualifierOperatorLike @selector(isLike:)

typedef BOOL (*BOOL_IMP)(id, SEL, ...);

// All qualifiers that can be evaluated in memory implement this protocol
@protocol FBQualifierEvaluation

- (BOOL)evaluateWithObject:(FBObject *)object; // returns YES if argument satifies the qualifier, NO otherwise.

@end

// All qualifiers that can be used to generate SQL queries implement this protocol
@protocol FBQualifierSQLGeneration

- (NSString *)sqlStringForSQLExpression:(FBSQLExpression *)sqlExpression;

- (FBQualifier *)schemaBasedQualifierWithRootEntity:(FBEntity *)entity;
// Returns an equivalent qualifier with object references
// replaced by foreign key references.

@end

// The following selectors are used when parsing an FBQualifier format.
// The following methods correspond to the following operators: =, <=, <, >=, >,
// !=, contains, and like.
/*@interface NSObject (FBRelationalSelectors)
- (BOOL)isEqualTo:(id)object;
    // Implemented using isEqual:. Returns NO if receiever is nil.
- (BOOL)isLessThanOrEqualTo:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)isLessThan:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)isGreaterThanOrEqualTo:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)isGreaterThan:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)isNotEqualTo:(id)object;
    // Implemented using compare. Returns NO if receiver is nil.
- (BOOL)doesContain:(id)object;
    // Returns nil if receiver is not an NSArray or if array doesn't contain object.
- (BOOL)isLike:(NSString *)object;
    // argument should be a string using simple shell wildcards (* and ?).
    // (e.g. "Stev*" or "N?XT").
    // Returns NO if receiver is not an NSString.
@end*/

@interface FBEntity (QualifierGeneration)

- (FBQualifier *)qualifierForObject:(FBObject *)object;
- (FBQualifier *)qualifierForGlobalID:(FBGlobalID *)globalID;

@end

@interface FBRelationship (QualifierGeneration)

- (FBQualifier *)destinationQualififerForObject:(FBObject *)object;

@end

/* **************************************************
   FBQualifier
   ************************************************** */
@interface FBQualifier : NSObject <FBQualifierEvaluation>
{
}

+ (NSDictionary *)selectorDic;
+ (FBQualifier *)qualifierWithQualifierString:(NSString *)string entity:(FBEntity *)entity;

// Used for parsing and printing of qualifiers
+ (NSString *)stringForOperatorSelector:(SEL)selector;
+ (SEL)operatorSelectorForString:(NSString *)string;

- (NSString *)sqlStringForSQLExpression:(FBSQLExpression *)sqlExpression;
- (BOOL)isComplexKeyQualifier;

@end

/* **************************************************
   FBKeyValueQualifier
   ************************************************** */
@interface FBKeyValueQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   SEL _selector;
   NSString *_key;
   id _value;
   BOOL _casesensitive;
   BOOL _invert;
}

+ (id)qualifierWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value;
+ (id)qualifierWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value casesensitive:(BOOL)flag;
+ (id)qualifierWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value invert:(BOOL)invert;
+ (id)qualifierWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value casesensitive:(BOOL)flag invert:(BOOL)invert;
- (id)initWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value;
- (id)initWithKey:(NSString *)key operatorSelector:(SEL)selector value:(id)value casesensitive:(BOOL)flag invert:(BOOL)invert;

- (BOOL)isCasesensitive;
- (BOOL)invert;

- (SEL)selector;

- (NSString *)key;

- (id)value;

@end

/* **************************************************
   FBKeyArrayQualifier
   ************************************************** */
@interface FBKeyArrayQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   BOOL _all;
   NSString *_key;
   NSArray *_array;
}

+ (id)qualifierWithKey:(NSString *)key and:(BOOL)flag array:(NSArray *)array;

- (id)initWithKey:(NSString *)key and:(BOOL)flag array:(NSArray *)array;

- (BOOL)all;

- (NSString *)key;

- (NSArray *)array;

@end

/* **************************************************
   FBSQLQualifier
   ************************************************** */
@interface FBSQLQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSString *_sql;
}

+ (id)qualifierWithSQL:(NSString *)sql;
- (id)initWithSQL:(NSString *)sql;

- (NSString *)sql;

@end

/* **************************************************
   FBInQualifier
   ************************************************** */
@interface FBInQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSString *_key;
   NSArray *_array;
}

+ (id)qualifierWithKey:(NSString *)key array:(NSArray *)array;

- (id)initWithKey:(NSString *)key array:(NSArray *)array;

- (NSString *)key;

- (NSArray *)array;

@end

/* **************************************************
   FBNotInQualifier
   ************************************************** */
@interface FBNotInQualifier : FBInQualifier
{
}

@end

/* **************************************************
   FBKeyComparisonQualifier
   ************************************************** */
@interface FBKeyComparisonQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
    SEL _selector;
    NSString *_leftKey;
    NSString *_rightKey;
}

+ (id)qualifierWithLeftKey:(NSString *)leftKey operatorSelector:(SEL)selector rightKey:(NSString *)rightKey;

- (id)initWithLeftKey:(NSString *)leftKey operatorSelector:(SEL)selector rightKey:(id)rightKey;

- (SEL)selector;

- (NSString *)leftKey;

- (NSString *)rightKey;

@end

/* **************************************************
   FBKeyIsNullQualifier
   ************************************************** */
@interface FBKeyIsNullQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSString *_key;
}

+ (id)qualifierWithKey:(NSString *)key;
- (id) initWithKey:(NSString *)key;
- (NSString *)key;

@end

/* **************************************************
   FBFalseQualifier
   ************************************************** */
@interface FBFalseQualifier : FBKeyIsNullQualifier

+ (id)qualifierForEntity:(FBEntity *)entity;

@end

/* **************************************************
   FBKeyIsNotNullQualifier
   ************************************************** */
@interface FBKeyIsNotNullQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSString *_key;
}

+ (id)qualifierWithKey:(NSString *)key;
- (id)initWithKey:(NSString *)key;
- (NSString *)key;

@end

/* **************************************************
   FBAndQualifier
   ************************************************** */
@interface FBAndQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration>
{
   NSMutableArray *_qualifiers;
}

+ (id)qualifierWithQualifierArray:(NSArray *)array;

- (id)initWithQualifierArray:(NSArray *)array;

- (NSArray *)qualifiers;

@end

/* **************************************************
   FBOrQualifier
   ************************************************** */
@interface FBOrQualifier : FBQualifier <FBQualifierEvaluation, FBQualifierSQLGeneration> 
{
    NSMutableArray *_qualifiers;
}

+ (id)qualifierWithQualifierArray:(NSArray *)array;

- (id)initWithQualifierArray:(NSArray *)array;

- (NSArray *)qualifiers;

@end

/* **************************************************
   FBKeyIsNullOrZeroQualifier
   ************************************************** */
@interface FBKeyIsNullOrZeroQualifier : FBOrQualifier
{
}

+ (id)qualifierWithKey:(NSString *)key;

@end

/* **************************************************
   FBIntermediateQualifier
   ************************************************** */
@interface FBIntermediateQualifier : FBQualifier <FBQualifierSQLGeneration>
{
   FBRelationship * _relationship;
   NSDictionary *_valueDic;
}

// This method is used by +[FBQualififer destinationQualifierForObject:] to qualify the target objects of a flattened relationship, e.g. Person -- addresses -->> Address over an intermediate table Person_addresses. This class is supposed to generate a where claus of the form "select * from Address where addressId in (select addressId from (Person_addresses natural join Address) where personId = ...)". The points are replaced with value. The joins are generated according to the relationships defined in the flattened relationship.
- (id)initWithFlattenedRelationship:(FBRelationship *)relationship valueDic:(NSDictionary *)valueDic;

- (FBRelationship *)relationship;

- (id)valueForKey:(NSString *)key;

@end

@interface NSString (NSStringQualifierExtension)

// returns YES if opening and closing bracktes summarize to zero
- (BOOL)isBalanced;

// removes leading and tailing spaces and bracktes
- (NSString *)cleanedString;
- (NSString *)stringAfterRemovingLeadingAndTailingSpaces;
- (NSString *)stringAfterRemovingLeadingAndTailingBrackets;

- (SEL)operatorSelectorForLeft:(NSString **)pLeft right:(NSString **)pRight;

- (id)valueForAttribute:(FBAttribute *)attribute;

@end
